@@ -2,26 +2,29 @@ import http.client
 import logging
 import os
 from base64 import b64decode
-from typing import List, Type
+from typing import List, TextIO, Type
 
 import click
 
 from ._version import version
+from .authenticator import BasicAuthenticator, OrgTokenAuthenticator, ProjectKeyAuthenticator
 from .chunk import MemfaultChunk
 from .context import MemfaultCliClickContext
 from .deploy import Deployer
 from .upload import (
     AndroidAppSymbolsUploader,
-    AospSymbolDirectoryUploader,
-    AospSymbolUploader,
     BugreportUploader,
     CoredumpUploader,
+    ElfSymbolDirectoryUploader,
+    ElfSymbolUploader,
     MarUploader,
     McuSdkElfSymbolUploader,
     ReleaseArtifactUploader,
     Uploader,
     XedUploader,
+    walk_files,
 )
+from .yocto import find_elf_files_from_image_manifest
 
 pass_memfault_cli_context = click.make_pass_decorator(MemfaultCliClickContext, ensure=True)
 click_argument_path = click.argument("path", type=click.Path(exists=True))
@@ -72,26 +75,32 @@ def cli(ctx: MemfaultCliClickContext, **kwargs):
 
 
 def _do_upload_or_raise(
-    ctx: MemfaultCliClickContext, path: str, uploader_cls: Type[Uploader]
+    ctx: MemfaultCliClickContext, path: str, uploader_cls: Type[Uploader], **kwargs
 ) -> None:
-    authenticator = ctx.authenticator
-    ctx.check_required_cli_args(authenticator=authenticator)
+    authenticator = ctx.create_authenticator(*uploader_cls.authenticator_types)
 
-    uploader = uploader_cls(ctx=ctx, file_path=path, authenticator=authenticator)
+    uploader = uploader_cls(ctx=ctx, file_path=path, authenticator=authenticator, **kwargs)
     if not uploader.can_upload_file():
         raise click.exceptions.UsageError("Upload failed!")
     uploader.upload()
 
 
 @cli.command(name="upload-coredump")
+@click.option(
+    "--device-serial",
+    required=False,
+    help="A fallback unique identifier of a device, when one is not present in the coredump",
+)
 @click_argument_path
 @pass_memfault_cli_context
-def upload_coredump(ctx: MemfaultCliClickContext, path: str):
+def upload_coredump(ctx: MemfaultCliClickContext, path: str, **kwargs):
     """Upload an MCU coredump for analysis.
 
     Coredumps can be added to an MCU platform by integrating the Memfault C SDK:
     https://github.com/memfault/memfault-firmware-sdk
     """
+    ctx.obj.update(**kwargs)
+
     _do_upload_or_raise(ctx, path, CoredumpUploader)
 
 
@@ -173,13 +182,16 @@ def upload_symbols(ctx: MemfaultCliClickContext, path: str, **kwargs):
 
     uploader: Type[Uploader]
     if os.path.isdir(path):
-        uploader = AospSymbolDirectoryUploader
+        uploader = ElfSymbolDirectoryUploader
+        uploader_kwargs = dict(file_paths=walk_files(path))
     elif ctx.software_info is None:
-        uploader = AospSymbolUploader
+        uploader = ElfSymbolUploader
+        uploader_kwargs = dict()
     else:
         uploader = McuSdkElfSymbolUploader
+        uploader_kwargs = dict()
 
-    _do_upload_or_raise(ctx, path, uploader)
+    _do_upload_or_raise(ctx, path, uploader, **uploader_kwargs)
 
 
 @cli.command(name="upload-mar")
@@ -211,6 +223,45 @@ def upload_mar(ctx: MemfaultCliClickContext, path: str, **kwargs):
     _do_upload_or_raise(ctx, path, MarUploader)
 
 
+@cli.command(name="upload-yocto-symbols")
+@click.option(
+    "--manifest",
+    "-m",
+    required=True,
+    help="""
+    The path to the image .manifest as produced by Yocto's do_rootfs (IMAGE_MANIFEST).
+    The .manifest file is expected to reside in the location where Yocto produced the file.
+    The tool will locate the symbol files to upload in the tmp/work/ directory, based on
+    the packages that are listed in the .manifest file.
+    """,
+    type=click.File(),
+)
+@click_option_concurrency
+@pass_memfault_cli_context
+def upload_yocto_symbols(ctx: MemfaultCliClickContext, manifest: TextIO, **kwargs):
+    """Upload symbols for a Yocto build.
+
+    To see a full example, take a look at the Linux SDK example project:
+    https://mflt.io/yocto-symbols-image
+
+    Example Yocto Symbol Upload:
+
+        \b
+        $ bitbake my-image
+        $ memfault --org-token $ORG_TOKEN \\
+                   --org acme-inc --project smart-sink \\
+            upload-yocto-symbols --manifest build/tmp/deploy/images/raspberrypi3/my-image-raspberrypi3.rootfs.manifest
+    """
+    ctx.obj.update(**kwargs)
+
+    ElfSymbolDirectoryUploader(
+        ctx=ctx,
+        file_path=manifest.name,  # Not used. TODO: refactor!
+        authenticator=ctx.create_authenticator(OrgTokenAuthenticator, BasicAuthenticator),
+        file_paths=find_elf_files_from_image_manifest(manifest),
+    ).upload()
+
+
 @cli.command(name="upload-mcu-symbols")
 @click.option(
     "--software-type",
@@ -286,11 +337,13 @@ def upload_aosp_symbols(ctx: MemfaultCliClickContext, path: str, **kwargs):
 
     uploader: Type[Uploader]
     if os.path.isdir(path):
-        uploader = AospSymbolDirectoryUploader
+        uploader = ElfSymbolDirectoryUploader
+        uploader_kwargs = dict(file_paths=walk_files(path))
     else:
-        uploader = AospSymbolUploader
+        uploader = ElfSymbolUploader
+        uploader_kwargs = dict()
 
-    _do_upload_or_raise(ctx, path, uploader)
+    _do_upload_or_raise(ctx, path, uploader, **uploader_kwargs)
 
 
 @cli.command(name="upload-android-app-symbols")
@@ -439,7 +492,7 @@ def deploy_release(ctx: MemfaultCliClickContext, cohort: str, rollout_percent: i
     """
     ctx.obj.update(**kwargs)
 
-    ctx.check_required_cli_args(authenticator=ctx.authenticator)
+    authenticator = ctx.create_authenticator(OrgTokenAuthenticator, BasicAuthenticator)
     ctx.check_required_either(
         {"release_version"}, {"delta_from", "delta_to"}, mutually_exclusive=True
     )
@@ -447,15 +500,12 @@ def deploy_release(ctx: MemfaultCliClickContext, cohort: str, rollout_percent: i
         ctx.obj["delta_from"],
         ctx.obj["delta_to"],
     )
-    deployer = Deployer(ctx=ctx, authenticator=ctx.authenticator)
+    deployer = Deployer(ctx=ctx, authenticator=authenticator)
     deployer.deploy(cohort=cohort, release_version=release_version, rollout_percent=rollout_percent)
 
 
 def _do_post_chunks_or_raise(ctx: MemfaultCliClickContext, chunks: List[bytes]):
-    authenticator = ctx.authenticator
-    ctx.check_required_cli_args(authenticator=authenticator)
-    if not authenticator.project_key_auth():
-        raise click.exceptions.UsageError("A '--project-key' is required")
+    authenticator = ctx.create_authenticator(ProjectKeyAuthenticator)
 
     MemfaultChunk(ctx, authenticator).batch_post(chunks)
     click.echo("Success")
